﻿using Microsoft.Data.Sqlite;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Hangman.Dejvoss
{
    /// <summary>
    /// SQLite Database Tools v0.0.2 | (c) DEJVOSS Productions 2024
    /// </summary>
    public class Database
    {
        SqliteConnection connection;
        SqliteCommand command;

        /// <summary>
        /// Constructor; connects to a database
        /// </summary>
        /// <param name="fileName">Filename of the db (database1.db)</param>
        public Database(String fileName)
        {
            connection = new SqliteConnection($"Data source={fileName}");
            connection.Open();
        }

        /// <summary>
        /// Creates a new table in the connected database
        /// </summary>
        /// <param name="name">Name of the table</param>
        /// <param name="items">Attributes of the database it will create; key = name, value = type</param>
        public void createTable(String name, Dictionary<String, String> items)
        {
            if (connection == null)
            {
                return;
            }

            command = connection.CreateCommand();
            command.CommandText = $"CREATE TABLE IF NOT EXISTS {name} (";
            for (int i = 0; i < items.Count; i++)
            {
                command.CommandText += $"{items.ElementAt(i).Key} {items.ElementAt(i).Value}";
                if (i != items.Count - 1)
                {
                    command.CommandText += ", ";
                }
            }

            command.CommandText += ");";
            command.ExecuteNonQuery();
            command.Dispose();
        }

        /// <summary>
        /// Returns one row from the table
        /// </summary>
        /// <param name="tableName">Name of query table</param>
        /// <param name="row">Number of query row</param>
        /// <returns></returns>
        public String[] returnFromTable(String tableName, int row)
        {
            if (connection == null)
            {
                return null;
            }

            command = connection.CreateCommand();

            command.CommandText = $"SELECT * from {tableName} ORDER BY ROWID ASC LIMIT {row}";

            var reader = command.ExecuteReader();

            //aa tohle je retardovany ale jinak to asi nejde

            String[] output = new String[6];
            while (reader.Read())
            {
                output[0] = reader.GetString(0);
                output[1] = reader.GetString(1);
                output[2] = reader.GetString(2);
                output[3] = reader.GetString(3);
                output[4] = reader.GetString(4);
                output[5] = reader.GetString(5);
            }

            command.Dispose();

            return output;
        }

        public List<String> readFromTable(String query, String tableName)
        {
            if (connection == null)
            {
                return null;
            }

            command = connection.CreateCommand();

            command.CommandText = $"SELECT {query} from {tableName}";

            var reader = command.ExecuteReader();

            List<String> output = new List<String>();
            do
            {
                while (reader.Read())
                {
                    output.Add(reader.GetString(0));
                }
            } while (reader.NextResult());

            command.Dispose();

            return output;
        }

        /// <summary>
        /// Inserts a row into the table
        /// </summary>
        /// <param name="tableName"></param>
        /// <param name="id"></param>
        /// <param name="subject"></param>
        /// <param name="teacher"></param>
        /// <param name="room"></param>
        /// <param name="group"></param>
        /// <param name="color"></param>
        public void insertIntoTable(String tableName, int id, String subject, String teacher, String room, String group, String color)
        {
            if (connection == null)
            {
                return;
            }

            command = connection.CreateCommand();
            command.CommandText = $"INSERT INTO {tableName} ('ID','SUBJECT','TEACHER','ROOM','GRUPPE','COLOR') values ({id},'{subject}','{teacher}', '{room}', '{group}', '{color}');";
            command.ExecuteNonQuery();
            command.Dispose();
        }

        public void deleteAllFromTable(String tableName)
        {
            command = connection.CreateCommand();
            command.CommandText = $"DELETE FROM {tableName};";
            command.ExecuteNonQuery();
            command.Dispose();
        }

        public void changeInTable(String tableName, int row, String query)
        {
            command = connection.CreateCommand();
            command.CommandText = $"SELECT {row} FROM {tableName};";
            command.ExecuteNonQuery();
            command.CommandText = $"UPDATE {tableName} SET {query};";
            command.ExecuteNonQuery();
            command.Dispose();
        }
    }
}
